<?php

/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
include 'env.php';
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Yappy extends PaymentModule
{
    protected $config_form = false;

    private function lookForUpdate(){
      
        try {
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, UPDATE_URL);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);        
            $result = curl_exec($curl);

           $response = json_decode($result, true);  
           if(version_compare( $this->version,$response['version'])==-1){

         
                return  '<div class="alert alert-success yappy-alert">
                <h4>'.$response['title'].'</h1>
                <p>'.$response['message'].'
                    <a target="_blank" href="'.$response['author_homepage'].'">Descargar actualización.</a></p>
                </div>';
                
           }
       
        } catch (\Throwable $th) {
           return false;
        }
       
    }

    public function __construct()
    {
        $this->name = 'Yappy';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.140';
        $this->author = 'Banco General';
        $this->need_instance = 0;

        /*
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();
        
        $this->displayName = $this->l('Botón de Pago Yappy');
        $this->description = $this->l('Utiliza Yappy como método de pago en tu tienda con el Botón de Pago Yappy de Banco General.') . '<br>' . $this->lookForUpdate();
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');

            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        // if (in_array($iso_code, $this->limited_countries) == false)
        // {
        //     $this->_errors[] = $this->l('This module is not available in your country');
        //     return false;
        // }

        Configuration::updateValue('YAPPY_LIVE_MODE', false);

        return parent::install()
        && $this->registerHook('header')
        && $this->registerHook('backOfficeHeader')
        && $this->registerHook('paymentOptions')
        && $this->registerHook('paymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('YAPPY_LIVE_MODE');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /*
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('submitYappyModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitYappyModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
        . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {

        return [
            'form' => [

                'legend' => [
                    'title' => $this->l('Configuración'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'YAPPY_ACCOUNT_MERCHANT',
                        'label' => $this->l('ID del comercio'),
                        'desc' => $this->l('Identificador único del comercio.'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'YAPPY_ACCOUNT_SECRET',
                        'label' => $this->l('Clave secreta'),
                        'desc' => $this->l('Clave alfanumérica necesaria para la conexión con Banco General.'),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Modo de pruebas'),
                        'name' => 'YAPPY_SANDBOX',
                        'is_bool' => true,
                        'desc' => $this->l('Cambiar entre modo de pruebas y ambiente productivo.'),
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('YES'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('NO'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Método de pago por defecto'),
                        'name' => 'YAPPY_DEFAULT_PAYMENT',
                        'is_bool' => true,
                        'desc' => $this->l('Habilita Yappy como la opción de pago por defecto, sin importar el orden.'),
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('YES'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('NO'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Habilitar logs'),
                        'name' => 'YAPPY_LOGS',
                        'is_bool' => true,
                        'desc' => $this->l('Recomendamos activarla solamente para depuración de errores o pruebas.'),
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('YES'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('NO'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'YAPPY_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return [
            'YAPPY_SANDBOX' => Configuration::get('YAPPY_SANDBOX', false),
            'YAPPY_LOGS' => Configuration::get('YAPPY_LOGS', false),
            'YAPPY_DEFAULT_PAYMENT' => Configuration::get('YAPPY_DEFAULT_PAYMENT', true),
            'YAPPY_LIVE_MODE' => Configuration::get('YAPPY_LIVE_MODE', true),
            'YAPPY_ACCOUNT_MERCHANT' => Configuration::get('YAPPY_ACCOUNT_MERCHANT', null),
            'YAPPY_ACCOUNT_SECRET' => Configuration::get('YAPPY_ACCOUNT_SECRET', null),
        ];
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();
        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addCSS($this->_path . 'views/css/global.css', 'all');
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS($this->_path . 'views/css/back.css', 'all');
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $defaultPayment = Configuration::get('YAPPY_DEFAULT_PAYMENT', null);
        if($defaultPayment=="1"){
            $this->context->controller->addJS($this->_path . '/views/js/default_payment.js');
        }
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int) $currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        return $this->display(__FILE__, 'views/templates/hook/payment_return.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_options = [
            $this->getExternalPaymentOption(),
        ];

        return $payment_options;
    }

    public function getExternalPaymentOption()
    {
        $externalOption = new PaymentOption();
        $externalOption->setAction($this->context->link->getModuleLink($this->name, 'redirect', [], true))
        ->setCallToActionText($this->l('Yappy'))
            ->setAdditionalInformation($this->context->smarty->fetch('module:Yappy/views/templates/front/payment_infos.tpl'))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/yappy-landscape.svg'));

        return $externalOption;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }
}
