<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
class YappyConfirmationModuleFrontController extends ModuleFrontController 
{
    public function postProcess()
    {
        $reference = Tools::getValue('orderId');
        $status = Tools::getValue('status');
        $hash = Tools::getValue('hash');
        $domain = Tools::getValue('domain');
        $confirmationNumber = Tools::getValue('confirmationNumber');

        if (isset($reference) && isset($status) && isset($domain) && isset($hash) && isset($confirmationNumber)) {
            header('Content-Type: application/json');

            $order_id = $this->getOrderByReference($reference);
            $success = $this->validateHash($reference, $status, $domain, $hash);

            if ($success) {
                switch ($status) {
                    case 'E':
                        $this->changeStatus($order_id, '2');
                        $this->createLog("Pago aceptado para orden con referencia: " .$reference, 1);
                        break;
                    case 'C':
                        $this->createLog("Pago cancelado para orden con referencia: " .$reference, 1);
                        $this->changeStatus($order_id, '6');
                        break;
                    case 'R':
                        $this->createLog("Pago rechazado para orden con referencia: " .$reference, 1);
                        $this->changeStatus($order_id, '6');
                        break;
                    default:
                        $this->createLog("Estado desconocido para orden con referencia: " .$reference, 1); 
                        $success = false;
                        break;
                }
            }
            echo json_encode(['succes' => $success]);
        } else {
            $this->createLog("Parametros de callback incompletos.", 3);
            echo json_encode(['succes' => false]);
        }
        $this->redirectWithNotifications('index.php?controller=order&step=1');
    }

    private function getOrderByReference($reference)
    {
        $result = Db::getInstance()->getRow('
        SELECT `id_order`
        FROM `' . _DB_PREFIX_ . 'orders`
        WHERE `reference` = "' . $reference . '"');

        return isset($result['id_order']) ? $result['id_order'] : false;
    }

    private function changeStatus($order_id, $status_id)
    {
        $objOrder = new Order((int) $order_id);
        $history = new OrderHistory();
        $history->id_order = (int) $objOrder->id;
        $history->changeIdOrderState($status_id, (int) ($objOrder->id));
        $history->add(true);
    }

    private function validateHash($reference, $status, $domain, $hash)
    {
        try {
            $secret = Configuration::get('YAPPY_ACCOUNT_SECRET', null);
            $values = base64_decode($secret);
            $secrete = explode('.', $values);
            $signature = hash_hmac('sha256', $reference . $status . $domain, $secrete[0]);
            $success = strcmp($hash, $signature) === 0;
        } catch (\Throwable $th) {
            $this->createLog("Algo salio mal al obtener respuesta de BG", 3);
            $success = false;
        }

        return $success;
    }

    private function createLog($message,$severity){
        $log = Configuration::get('YAPPY_LOGS', null);
        if($log=="1"){
            PrestaShopLogger::addLog("[Botón de pago Yappy] " . $message,$severity);
        }
    }
}
