<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
require_once _PS_MODULE_DIR_ . 'Yappy/Bg/BgFirma.php';
use Bg\BgFirma;

class YappyRedirectModuleFrontController extends ModuleFrontController 
{
    const DOMAIN_REGEX = '/^(https:\/\/www\.|https:\/\/)?[a-zñ0-9]+([\-\.]{1}[a-zñ0-9]+)*\.[a-z]{2,10}(:[0-9]{1,5})?(\/.*)?$/';
    private $bgFirma;

    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        /*
         * Oops, an error occured.
         */
        

        if (Tools::getValue('action') == 'error') {
            return $this->displayError('An error occurred while trying to redirect the customer');
           
        } else {
            $this->context->smarty->assign([
                'cart_id' => Context::getContext()->cart->id,
                'secure_key' => Context::getContext()->customer->secure_key,
            ]);
            $merchant = Configuration::get('YAPPY_ACCOUNT_MERCHANT', null);
            $secret = Configuration::get('YAPPY_ACCOUNT_SECRET', null);
            $sandbox = Configuration::get('YAPPY_SANDBOX', null) == '1' ? true : false;

            try {
                $protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? 'https://' : 'http://';
                $domain = $protocol . $_SERVER['HTTP_HOST'];
                $response = $this->checkCredentials($merchant, $secret, $domain); //cambiar por $domain

                if ($response && $response['success']) {
                    $this->createLog("Credenciales correctas.",1);
                    $cart = $this->context->cart;
                    $message = null; // You can add a comment directly into the order so the merchant will see it in the BO
                    $module_name = $this->module->displayName;
                    $currency_id = (int) Context::getContext()->currency->id;
                    $secure_key = Context::getContext()->customer->secure_key;
                    $payment_status = Configuration::get('PS_OS_BANKWIRE');
                    $protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? 'https://' : 'http://';
                    $summary = $this->context->cart->getSummaryDetails();
                    $address = new Address($cart->id_address_delivery);


                    

                    if(!$sandbox){
                        $this->module->validateOrder($cart->id, $payment_status, $cart->getOrderTotal(), $module_name, $message, [], $currency_id, false, $secure_key);
                        $order_id = Order::getOrderByCartId($cart->id);
                        $order = new Order((int) $order_id);
                        $reference = $order->reference;
                        $succesUrl = $this->context->link->getPageLink('index', true) . 'order-confirmation?id_cart=' . (int) $cart->id . '&id_module=' . (int) $this->module->id . '&order_id=' . $this->module->currentOrder . '&key=' . $secure_key;
                    }else{
                        $reference = "pruebaPago";
                        $succesUrl = $domain;
                    }
                    

                    

                    $paymentarray = [
                        'total' => $summary['total_price'],
                        'subtotal' => $summary['total_price_without_tax'],
                        'taxes' => $summary['total_tax'],
                        'discount' => $summary['total_discounts'],
                        'shipping' => $summary['total_shipping'],
                        'orderId' => $reference,
                        'secret' => $secret,
                        'merchant' => $merchant,
                        'phone' => $address->phone,
                        'sandbox' => $sandbox,
                    ];

                    $this->bgFirma = new BgFirma($paymentarray, $response, $succesUrl, $domain, $domain);
                    $checkout = $this->bgFirma->generateCheckout();

                    if ($checkout && $checkout['success']) {
                        $this->createLog("Redireccionando a sitio seguro BG.",1);
                        Tools::redirect($checkout['url']);
                    } else {
                        $this->createLog("Ocurrio un error al tratar de redireccionar al cliente.",3);
                        $this->errors[] = $this->l('Algo salió mal. Contacta al administrador.');
                        $this->redirectWithNotifications('index.php?controller=order&step=1');
                    }
                } else {
                    $this->createLog("Credenciales incorrectas.",3);
                    $this->errors[] = $this->l('Algo salió mal. Contacta al administrador.');
                    $this->redirectWithNotifications('index.php?controller=order&step=1');
                }
            } catch (Exception $e) {
                $this->createLog( $e->getMessage(),4);
                $this->errors[] = $this->l('Algo salió mal. Contacta al administrador.');
                $this->redirectWithNotifications('index.php?controller=order&step=1');
            }
        }
    }

    protected function displayError($message, $description = false)
    {
        /*
         * Create the breadcrumb for your ModuleFrontController.
         */
        $this->context->smarty->assign('path', '
			<a href="' . $this->context->link->getPageLink('order', null, null, 'step=3') . '">' . $this->module->l('Payment') . '</a>
			<span class="navigation-pipe">&gt;</span>' . $this->module->l('Error'));

        /*
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message), $description);

        return $this->setTemplate('module:Yappy/views/templates/front/error.tpl');
    }



    private function checkCredentials($merchantId, $secretKey, $domain)
    {
        try {
            if (!preg_match(self::DOMAIN_REGEX, $domain) || strlen($domain) > 64) {
                return ['success' => false];
            }
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, API_URL . '/validateapikeymerchand');
            curl_setopt($curl, CURLOPT_POST, 1);
            $values = base64_decode($secretKey);
            $secrete = explode('.', $values);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_HTTPHEADER, [
                'x-api-key: ' . $secrete[1],
                'Content-Type: application/json',
            ]);

            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode([
                'merchantId' => $merchantId,
                'urlDomain' => $domain,
            ]));
            $result = curl_exec($curl);
            $response = json_decode($result, true);
            if (isset($response['unixTimestamp'])) {
                $response['unixTimestamp'] = $response['unixTimestamp'] * 1000;
            }

            curl_close($curl);

            return $response;
        } catch (\Throwable $th) {
            return ['success' => false];
        }
    }

    private function createLog($message,$severity){
        $log = Configuration::get('YAPPY_LOGS', null);
        if($log=="1"){
            PrestaShopLogger::addLog("[Botón de pago Yappy] " . $message,$severity);
        }
    }
}
